use std::iter::Iterator;

pub struct CharScore {
    table: [f32; 256],
}

impl CharScore {
    pub fn new() -> Self {
        let letter_score: [f32; 26] = [
            81.67, 14.92, 27.82, 42.53, 127.02, 22.28, 20.15, 6.094, 69.66, 1.53, 7.72, 40.25,
            24.06, 67.49, 75.07, 19.29, 0.95, 59.87, 63.27, 90.56, 27.58, 9.78, 23.60, 1.50, 19.74,
            0.74,
        ];
        let digit_score: f32 = 0.005;
        let punct_score: f32 = 0.001;
        let blank_score: f32 = 0.;

        let mut table: [f32; 256] = [-1.; 256];
        for byte in 0..=255 {
            let i = usize::from(byte);
            let c: char = char::from(byte);
            if c.is_ascii_alphabetic() {
                if c.is_ascii_uppercase() {
                    table[i] = (letter_score[usize::from(byte - 65u8)] / 1000f32).sqrt();
                } else {
                    table[i] = (letter_score[usize::from(byte - 97u8)] / 1000f32).sqrt();
                }
            } else if c.is_ascii_digit() {
                table[i] = digit_score;
            } else if c.is_ascii_punctuation() {
                table[i] = punct_score;
            } else if c.is_ascii_whitespace() {
                table[i] = blank_score;
            }
        }
        Self { table }
    }

    pub fn score_char(&self, c: &u8) -> f32 {
        self.table[usize::from(*c)]
    }

    pub fn score<'a, I>(&self, data: I) -> f32
    where
        I: Iterator<Item = &'a u8>,
    {
        data.fold(0., |acc, x| acc + self.score_char(x))
    }
}
